/*
 * vim: ts=4 nobackup
 */


#include "util.H"
#include "PWM.H"
#include "Chronometer.H"
#include "RPMReader.H"
#include "ADCManager.H"
#include "Fuzzy.H"
#include <avr/io.h>
#include <avr/interrupt.h>


using namespace std;
using namespace avelino;


#define  SET_POINT       3000               // rpm
#define  SET_POINT_LOW   (SET_POINT - 200)
#define  SET_POINT_HIGH  (SET_POINT + 200)


namespace avelino {
	class RPMErrorFuzzyValueReader : public FuzzyValueReader {
		protected:
			float setPoint;
		public:
			RPMErrorFuzzyValueReader(float s);
			float getValue();
	};

	class SpeedDeltaFuzzyValueWriter : public FuzzyValueWriter {
		protected:
			float sum;
		public:
			SpeedDeltaFuzzyValueWriter();
			void setValue(float v);
	};
}


RPMErrorFuzzyValueReader::RPMErrorFuzzyValueReader(float s) {
	this->setPoint = s;
}


float RPMErrorFuzzyValueReader::getValue() {
	return (this->setPoint - ((float) RPMReader::rpm));
}


SpeedDeltaFuzzyValueWriter::SpeedDeltaFuzzyValueWriter() {
	this->sum = 0;
}


void SpeedDeltaFuzzyValueWriter::setValue(float v) {
	this->sum += v;
	if (this->sum > 255)
		this->sum = 255;
	else if (this->sum < 0)
		this->sum = 0;
	uint8_t aux = (uint8_t) this->sum;
	PWM::setPB7Value(aux);
}


RPMErrorFuzzyValueReader rpmError(SET_POINT);
SpeedDeltaFuzzyValueWriter speedDelta;
FuzzySet negativeError(-10000, -10000, -100, 0);
FuzzySet noError(-100, 0, 0, 100);
FuzzySet positiveError(0, 100, 10000, 10000);
FuzzySet decreaseSpeed(-20, -20, -20, 0);
FuzzySet keepSpeed(-20, 0, 0, 20);
FuzzySet increaseSpeed(0, 20, 20, 20);
FuzzyController c;


void disableUSBModule() {
	cli();
	PRR1 |= 1 << PRUSB;
	sei();
}


int main() {
	disableUSBModule();
	ADCManager::init();
	PWM::init();
	Chronometer::init();
	RPMReader::init();
	DDRC |= 0x80;
	// define fuzzy logic
	c.addRule(rpmError % negativeError, new FuzzyConsequent(speedDelta, decreaseSpeed));
	c.addRule(rpmError % noError,       new FuzzyConsequent(speedDelta, keepSpeed));
	c.addRule(rpmError % positiveError, new FuzzyConsequent(speedDelta, increaseSpeed));
	uint32_t prevT = 0;
	while (true) {
		if (Chronometer::microseconds2 > 200000) {    // each 200ms
			Chronometer::microseconds2 = 0;
			c.run();
		}
		/*
		uint16_t v = ADCManager::get(0);
		PWM::setPB7Value(v >> 2);
		*/
		if ((RPMReader::rpm >= SET_POINT_LOW) && (RPMReader::rpm <= SET_POINT_HIGH))
			PORTC |= 0x80;
		else
			PORTC &= 0x7F;
	}
	return 0;
}
